@@ -10,7 +10,13 @@
 #' @param end_date string End date to retrieve data from, format 'yyyymmdd', if not provided, today will be assumed
 #' @param interval string Interval with which to sample data according to what `seq()` needs
 #' @param quote logical set to TRUE if you want to include price data (FALSE=default)
+#' @param sort string use to sort results, possible values: "name", "symbol", "market_cap", "price",
+#' "circulating_supply", "total_supply", "max_supply", "num_market_pairs", "volume_24h",
+#' "volume_7d", "volume_30d", "percent_change_1h", "percent_change_24h",
+#' "percent_change_7d". Especially useful if you only want to download the top x entries using "limit" (deprecated for "new")
+#' @param sort_dir string used to specify the direction of the sort in "sort". Possible values are "asc" (DEFAULT) and "desc"
 #' @param sleep integer (default 60) Seconds to sleep between API requests
+#' @param wait waiting time before retry in case of fail (needs to be larger than 60s in case the server blocks too many attempts, default=60)
 #' @param finalWait to avoid calling the web-api again with another command before 60s are over (TRUE=default)
 #'
 #' @return List of latest/new/historic listings of cryptocurrencies in a tibble (depending on the "which"-switch and
@@ -61,18 +67,23 @@
 #' # report in two different currencies
 #' listings_2014w1_USDBTC <- crypto_listings(which="historical", quote=TRUE,
 #' start_date = "20140101", end_date="20140107", interval="day", convert="USD,BTC")
+#'
+#' # only download the top 10 crypto currencies based on their market capitalization
+#' listings_2015w1_mcsort <- crypto_listings(which="historical", quote=TRUE,
+#' start_date = "20150101", end_date="20150107", interval="day", sort="market_cap",
+#' sort_dir="desc", limit=10)
 #' }
 #'
 #' @name crypto_listings
 #'
 #' @export
 #'
-crypto_listings <- function(which="latest", convert="USD", limit = 5000, start_date = NULL, end_date = NULL, interval = "day", quote=FALSE, sleep = 0, finalWait = FALSE) {
+crypto_listings <- function(which="latest", convert="USD", limit = 5000, start_date = NULL, end_date = NULL, interval = "day", quote=FALSE, sort="market_cap", sort_dir="desc", sleep = 0, wait = 60, finalWait = FALSE) {
   # get current coins
   listing_raw <- NULL
   if (which=="new"){
     for (i in 1:10){
-      new_url <- paste0("https://web-api.coinmarketcap.com/v1/cryptocurrency/listings/new?limit=",limit,"&convert=",convert,"&start=",(i-1)*5000+1)
+      new_url <- paste0("https://web-api.coinmarketcap.com/v1/cryptocurrency/listings/new?limit=",limit,"&convert=",convert,"&sort_dir=",sort_dir,"&start=",(i-1)*5000+1)
       new_raw <- jsonlite::fromJSON(new_url)
       listing_raw <- bind_rows(listing_raw,
                                new_raw$data %>% tibble::as_tibble() %>% dplyr::mutate(dplyr::across(c(date_added,last_updated),as.Date)) %>%
@@ -87,7 +98,7 @@ crypto_listings <- function(which="latest", convert="USD", limit = 5000, start_d
   } else if (which=="latest"){
     for (i in 1:10){
       latest_url <- paste0("https://web-api.coinmarketcap.com/v1/cryptocurrency/listings/latest?limit=",limit,
-                        "&aux=market_cap_by_total_supply&convert=",convert,"&start=",(i-1)*5000+1)
+                        "&aux=market_cap_by_total_supply&convert=",convert,"&sort=",sort,"&sort_dir=",sort_dir,"&start=",(i-1)*5000+1)
       latest_raw <- jsonlite::fromJSON(latest_url)
       listing_raw <- bind_rows(listing_raw,
                                latest_raw$data %>% tibble::as_tibble() %>% dplyr::mutate(dplyr::across(c(last_updated),as.Date)) %>%
@@ -107,7 +118,7 @@ crypto_listings <- function(which="latest", convert="USD", limit = 5000, start_d
     dates <- seq(sdate, edate, by=interval)
     tbdate <- enframe(dates[which(dates<Sys.Date())],name=NULL) %>% rename(date=value) %>%
       mutate(historyurl = paste0("https://web-api.coinmarketcap.com/v1/cryptocurrency/listings/historical?date=",dates,
-                                 "&limit=",limit,"&sort=cmc_rank&sort_dir=asc&convert=",convert,"&start="))
+                                 "&limit=",limit,"&convert=",convert,"&sort=",sort,"&sort_dir=",sort_dir,"&start="))
     # scraping tools
     scrape_web <- function(historyurl,quote){
       listing_raw <- NULL
@@ -128,7 +139,7 @@ crypto_listings <- function(which="latest", convert="USD", limit = 5000, start_d
       return(listing)
     }
     # define backoff rate
-    rate <- purrr::rate_delay(pause = 60,max_times = 2)
+    rate <- purrr::rate_delay(pause = wait, max_times = 2)
     rate2 <- purrr::rate_delay(sleep)
     #rate_backoff(pause_base = 3, pause_cap = 70, pause_min = 40, max_times = 10, jitter = TRUE)
     # Modify function to run insistently.
